#ifndef __LIBWR2RF__API__H_
#define __LIBWR2RF__API__H_

#include "io.h"

/* Using libwr2rf:

   1. Get an handle to the board using libwr2rf_host_init_by_slot from host.h.
   2. Check the board is present using libwr2rf_check_hwversion.
   3. Enable and check the VCXO.  If necessary, wait until the temperature
      is stabilized.  Use libwr2rf_enable_check_ocxo.
   4. Initialize pll, clocks: libwr2rf_clock_init
   5. TBD: Load WRC firmware if necessary
   6. Check WRC link: libwr2rf_check_link_time
   7. DDS init + DAC init: libwr2rf_rf_init
   8. Enable rf-out output: libwr2rf_set_rf_out

   20. Configure diag units
   21. Configure trigger units
*/

/* Read FW version.  */
unsigned libwr2rf_read_fwversion(struct libwr2rf_dev *dev);

/* Check presence of the board and FW version.
   Returns 0 in case of success.  */
int libwr2rf_check_fwversion(struct libwr2rf_dev *dev);

struct libwr2rf_nco_ipinfo_t {
    uint32_t std_ver;
    uint32_t ident;
    uint32_t fw_ver;
    uint32_t mem_map_ver;
};

/* Return ipinfo for NCO.  */
void libwr2rf_read_nco_ipinfo(struct libwr2rf_dev *dev,
                             struct libwr2rf_nco_ipinfo_t* pinfo);

/* Enable OCXO and return the remaining warmup time in seconds.
   Return 0 if already warmed.  */
int libwr2rf_enable_check_ocxo(struct libwr2rf_dev *dev);

/* Initialize PLL, enable WR.  */
void libwr2rf_clock_init(struct libwr2rf_dev *dev);

/* Return iff WR time is valid.  */
int libwr2rf_wrc_time_valid(struct libwr2rf_dev *dev);

/* Return true if both the WR link and time are OK.  */
int libwr2rf_check_link_time(struct libwr2rf_dev *dev);

/* Read the mac address (6 bytes).  Can be used as unique id.  */
int libwr2rf_get_mac(struct libwr2rf_dev *dev, unsigned char *mac);

/* DAC IQ set points.  */
int libwr2rf_dac_iqsetpoint(struct libwr2rf_dev *dev, unsigned ch, unsigned igain, unsigned qgain);

/* Mask (or not mask) NCO resets.  */
int libwr2rf_nco_reset(struct libwr2rf_dev *dev, unsigned ch, unsigned en);

/* RF-out SMA connectors.  */
#define LIBWR2RF_RF_OUT_GND	0
#define LIBWR2RF_RF_OUT_MIXER	1
#define LIBWR2RF_RF_OUT_DDS	2
int libwr2rf_set_rf_out(struct libwr2rf_dev *dev, unsigned ch, unsigned cfg);

/* Initialize RF (DDS and DAC).  */
int libwr2rf_rf_init (struct libwr2rf_dev *dev);

/* Post WR full initialization.  */
int libwr2rf_api_init (struct libwr2rf_dev *dev);

/* tmg-io connector.  */
//#define LIBWR2RF_TMGIO_OUT_RFSYNC      0  /* For channels 1 & 2  */
//#define LIBWR2RF_TMGIO_OUT_FREV        1  /* For channels 1 & 2  */
//#define LIBWR2RF_TMGIO_OUT_FREV_MAIN   2  /* For channels 1 & 2  */
int libwr2rf_configure_tmgio(struct libwr2rf_dev *dev, unsigned lemo,
                             unsigned oe, unsigned term);
int libwr2rf_configure_tmgclk(struct libwr2rf_dev *dev, unsigned lemo,
                              unsigned oe, unsigned term);


/* Diagnostic units.  */
#define LIBWR2RF_DIAG_ID_RF_1        0
#define LIBWR2RF_DIAG_ID_RF_2        1
#define LIBWR2RF_DIAG_ID_RF_1_TRIG_1 2
#define LIBWR2RF_DIAG_ID_RF_1_TRIG_2 3
#define LIBWR2RF_DIAG_ID_RF_2_TRIG_1 4
#define LIBWR2RF_DIAG_ID_RF_2_TRIG_2 5

/* Configure and enable diagnostic unit ID.
   WINDOW_US is the time window (in microsec) used to compute the frequency.
   Available values are 5us, 500us, 50ms, 5s, 20s.
   GENERATION returns the generation value for the previous configuration.
   So results for this generation are not correct.
   Returns 0 in case of success.  Possible errors are bad ID or bad WINDOW_US.
*/
int libwr2rf_enable_diag(struct libwr2rf_dev *dev, unsigned id,
                         unsigned window_us, unsigned *generation);

/* Read diagnostic unit ID.
   FREQ is the number of pulses for the previous generation, while COUNT is
   the total number of pulses since the board started.
   So to compute the real frequency, you need to divide by the window period.
   GENERATION is incremented after each window.
   When you change the configuration, you have to skip 2 generations: skip
   one for the results of the previous config, and skip another one to get
   the first result of the new config.  */
int libwr2rf_read_diag(struct libwr2rf_dev *dev, unsigned id,
                       unsigned *freq, unsigned *count, unsigned *generation);

#define LIBWR2RF_RF1_CHANNEL_ID 1
#define LIBWR2RF_RF2_CHANNEL_ID 2

/* Trigger units.  */
#define LIBWR2RF_TRIG_ID_RF_1_TRIG_1 0
#define LIBWR2RF_TRIG_ID_RF_1_TRIG_2 1
#define LIBWR2RF_TRIG_ID_RF_2_TRIG_1 2
#define LIBWR2RF_TRIG_ID_RF_2_TRIG_2 3

/* LEMO names.  */
#define LIBWR2RF_LEMO_TMGIO1 1
#define LIBWR2RF_LEMO_TMGIO2 2
#define LIBWR2RF_LEMO_TMGIO3 3
#define LIBWR2RF_LEMO_TMGIO4 4
#define LIBWR2RF_LEMO_TMGCLK1 5
#define LIBWR2RF_LEMO_TMGCLK2 6

/* Debug names */
#define LIBWR2RF_LEMO_DBG_NCO_RESET              1
#define LIBWR2RF_LEMO_DBG_RF1_NCO_RESET_CDELAYED 2
#define LIBWR2RF_LEMO_DBG_RF2_NCO_RESET_CDELAYED 3
#define LIBWR2RF_LEMO_DBG_RF1_FREV               4
#define LIBWR2RF_LEMO_DBG_RF1_FREV_MAIN          5
#define LIBWR2RF_LEMO_DBG_RF2_FREV               6
#define LIBWR2RF_LEMO_DBG_RF2_FREV_MAIN          7
#define LIBWR2RF_LEMO_DBG_WRS_RX_VALID           8
#define LIBWR2RF_LEMO_DBG_WRS_TX_VALID           9

/* LEMO front panel debug selection */
int libwr2rf_configure_lemo_debug(struct libwr2rf_dev *dev, unsigned lemo, unsigned dbg);


/* Trigger unit programmation.
   The functions always set the offline configuration.  */

/* Select which lemo is used for start/stop input for trigger ID.  */
int libwr2rf_vtu_start_lemo(struct libwr2rf_dev *dev, unsigned id,
                            unsigned lemo);
int libwr2rf_vtu_stop_lemo(struct libwr2rf_dev *dev, unsigned id,
                            unsigned lemo);

/* Put VTU in/out of reset.  */
int libwr2rf_vtu_reset (struct libwr2rf_dev *dev, unsigned id, unsigned rst);

/* Software stop/start of a VTU. */
int libwr2rf_vtu_softstop (struct libwr2rf_dev *dev, unsigned id);
int libwr2rf_vtu_softstart (struct libwr2rf_dev *dev, unsigned id);

/* Program the delay line of a VTU. VAL is the delay (TBD).  */
int libwr2rf_vtu_output_ff_delay(struct libwr2rf_dev *dev, unsigned id,
                              unsigned val);

int libwr2rf_vtu_output_pad_delay(struct libwr2rf_dev *dev, unsigned id,
                                  unsigned hc, unsigned odelay);

/* Enable the VTU output.  */
int libwr2rf_vtu_output_enable(struct libwr2rf_dev *dev, unsigned id,
                               unsigned en);

/* Enable the VTU pulse shaper.  */
int libwr2rf_vtu_output_shaper(struct libwr2rf_dev *dev, unsigned id,
                               unsigned en);

/* Hold trigger output in reset or not.  */
int libwr2rf_vtu_output_reset(struct libwr2rf_dev *dev, unsigned id,
                              unsigned rst);

/* Program a single pulse on TU ID after B clock cycles.  */
int libwr2rf_vtu_program_pulse (struct libwr2rf_dev *dev, unsigned id,
                                uint64_t b);

/* Program periodic pulses (evey HT clock cycles) after B clock cycles.  */
int libwr2rf_vtu_program_infinite (struct libwr2rf_dev *dev, unsigned id,
                                   uint64_t b, unsigned ht);

/* Program high frequency division (factor DIV) after B clock cycles.  */
int libwr2rf_vtu_program_highfreq (struct libwr2rf_dev *dev, unsigned id,
                                   uint64_t b, unsigned div);

/* Program W periodic pulses (evey HT clock cycles) after B clock cycles.  */
int libwr2rf_vtu_program_window (struct libwr2rf_dev *dev, unsigned id,
                                 uint64_t b, unsigned ht, unsigned w);

/* Invalidate the programmation.  If the programmation was already applied
   (with a start pulse of a softstart), this has no effect.  */
int libwr2rf_vtu_program_invalidate (struct libwr2rf_dev *dev, unsigned id);

/* Specify the delay between the NCO reset (from WR) and the sync input of
   trig 1.  CDELAY unit is 16ns, FDELAY unit is 1ns.  */
int libwr2rf_vtu_nco_reset_delay (struct libwr2rf_dev *dev, unsigned id,
                                  unsigned cdelay, unsigned fdelay,
                                  unsigned odelay);

/* Dump (on stdout) the state of VTU ID.  Return < 0 in case of error (bad ID).
   This is a debug function.  */
int libwr2rf_vtu_dump (struct libwr2rf_dev *dev, unsigned id);

/* Specify the ioupdate fine delay and ODELAY values */
int libwr2rf_ioupdate_delay (struct libwr2rf_dev *dev, unsigned fdelay, unsigned odelay);

/* Set WR streamer fixed latency: the delay between the sent time and the apply
   time (in WR cycles ie 16ns).  */
void libwr2rf_set_wrs_fixed_latency (struct libwr2rf_dev *dev,
                                     unsigned wr_cycles);
unsigned libwr2rf_get_wrs_fixed_latency (struct libwr2rf_dev *dev);

/* Set WR streamer timeout latency: the maximum time the packet arrive in advance.
   Too early coming packets are discarded.  */
void libwr2rf_set_wrs_timeout_latency (struct libwr2rf_dev *dev, unsigned wr_cycles);
unsigned libwr2rf_get_wrs_timeout_latency (struct libwr2rf_dev *dev);

#endif /* __LIBWR2RF__API__H_ */
